
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: IsAddOnLoaded, LibStub, next

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration AdiBags")
local Masque = LibStub('Masque', true)

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self.containerFrames = {}
	self.itemButtons = {}
end

function M:OnEnable()
	if not IsAddOnLoaded("AdiBags") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	self.AdiBags = LibStub("AceAddon-3.0"):GetAddon("AdiBags")
	self.containerClass = self.AdiBags:GetClass("Container")
	self.buttonClass = self.AdiBags:GetClass("ItemButton")
	self.buttonClass.metatable.__index.masqueGroup = Masque:Group("AdiBags", "Bank")
	self.buttonClass.metatable.__index.masqueGroup = Masque:Group("AdiBags", "Backpack")

	if not self:IsHooked(self.containerClass.metatable.__index, "OnCreate") then
		self:SecureHook(self.containerClass.metatable.__index, "OnCreate", "Container_OnCreate")
	end

	if not self:IsHooked(self.buttonClass.metatable.__index, "OnCreate") then
		self:SecureHook(self.buttonClass.metatable.__index, "OnCreate", "ItemButton_OnCreate")
	end

	if not self:IsHooked(self.buttonClass.metatable.__index, "UpdateBorder") then
		self:SecureHook(self.buttonClass.metatable.__index, "UpdateBorder", "ItemButton_UpdateBorder")
	end

	self:EnableBorders()
end

function M:OnDisable()
	self:DisableBorders()
end

-- Enable SixUI's borders on all known item buttons.
function M:EnableBorders()
	for containerFrame in next, self.containerFrames do
		containerFrame:GetBorder():Show()
	end

	for itemButton in next, self.itemButtons do
		self:StyleItemButton(itemButton)
	end
end

-- Disable SixUI's borders on all known item buttons.
function M:DisableBorders()
	for containerFrame in next, self.containerFrames do
		containerFrame:GetBorder():Hide()
	end

	for itemButton in next, self.itemButtons do
		itemButton:GetBorder():Hide()
	end
end

-- Apply SixUI's style to a container frame.
function M:StyleContainerFrame(containerFrame)
	B:CreateBorder(containerFrame)
	self.containerFrames[containerFrame] = true
end

-- Apply SixUI's style to an item button.
function M:StyleItemButton(itemButton)
	B:CreateBorder(itemButton, C.BORDER_PADDING):SetBackground(false)
	self:UpdateBorder(itemButton)
	self.itemButtons[itemButton] = true

	if Masque then
		itemButton.masqueData = {
			FloatingBG = false,
			Icon = itemButton.IconTexture,
			Cooldown = itemButton.Cooldown,
			Flash = false,
			Pushed = false,
			Normal = itemButton.NormalTexture,
			Disabled = false,
			Checked = false,
			Border = itemButton.IconQuestTexture,
			AutoCastable = false,
			HotKey = itemButton.Stock,
			Count = itemButton.Count,
			Name = false,
			Duration = false,
			AutoCast = false,
		}
	end
end

-- Synchronise SixUI's border color with AdiBags.
function M:UpdateBorder(itemButton)
	local r, g, b = itemButton.IconQuestTexture:GetVertexColor()
	itemButton:GetBorder():SetColor(r, g, b)
	itemButton.IconQuestTexture:Hide()

	if Masque then
		itemButton.masqueGroup:RemoveButton(itemButton)
		itemButton.masqueGroup:AddButton(itemButton, itemButton.masqueData)
	end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:ADDON_LOADED(event, name)
	if name == "AdiBags" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnEnable()
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

-- Use a hook to find newly created container frames.
function M:Container_OnCreate(_, name, isBank, bagObject)
	if not self:IsHooked(bagObject, "GetFrame") then
		self:SecureHook(bagObject, "GetFrame")
	end
end

-- Use a hook to find newly created bag frames.
function M:GetFrame(bagObject)
	if bagObject and bagObject:HasFrame() then
		self:StyleContainerFrame(bagObject.frame)
	end
end

-- Use a hook to find newly created item buttons.
function M:ItemButton_OnCreate(itemButton)
	self:StyleItemButton(itemButton)
end

-- Use a hook to synchronise SixUI's border color with Adibags.
function M:ItemButton_UpdateBorder(itemButton)
	self:StyleItemButton(itemButton)
end
